@@ -35,7 +35,7 @@ convenience functions to create required data format from raw data
 
 """
 
-
+from __future__ import division
 import numpy as np
 from scipy import stats  #get rid of this? need only norm.sf
 
@@ -46,7 +46,7 @@ class ResultsBunch(dict):
 
     def __init__(self, **kwds):
         dict.__init__(self, kwds)
-        self.__dict__  = self
+        self.__dict__ = self
         self._initialize()
 
     def _initialize(self):
@@ -192,27 +192,57 @@ def to_table(data, bins=None):
 
     return tt[0], bins_
 
-def fleiss_kappa(table):
-    '''Fleiss' kappa multi-rater agreement measure
+def fleiss_kappa(table, method='fleiss'):
+    """Fleiss' and Randolph's kappa multi-rater agreement measure
 
     Parameters
     ----------
     table : array_like, 2-D
         assumes subjects in rows, and categories in columns
+    method : string
+        Method 'fleiss' returns Fleiss' kappa which uses the sample margin
+        to define the chance outcome.
+        Method 'randolph' or 'uniform' (only first 4 letters are needed)
+        returns Randolph's (2005) multirater kappa which assumes a uniform
+        distribution of the categories to define the chance outcome.
 
     Returns
     -------
     kappa : float
-        Fleiss's kappa statistic for inter rater agreement
+        Fleiss's or Randolph's kappa statistic for inter rater agreement
 
     Notes
     -----
-    coded from Wikipedia page
-    http://en.wikipedia.org/wiki/Fleiss%27_kappa
+    no variance or hypothesis tests yet
 
-    no variance or tests yet
+    Interrater agreement measures like Fleiss's kappa measure agreement relative
+    to chance agreement. Different authors have proposed ways of defining
+    these chance agreements. Fleiss' is based on the marginal sample distribution
+    of categories, while Randolph uses a uniform distribution of categories as
+    benchmark. Warrens (2010) showed that Randolph's kappa is always larger or
+    equal to Fleiss' kappa. Under some commonly observed condition, Fleiss' and
+    Randolph's kappa provide lower and upper bounds for two similar kappa_like
+    measures by Light (1971) and Hubert (1977).
+
+    References
+    ----------
+    Wikipedia http://en.wikipedia.org/wiki/Fleiss%27_kappa
+
+    Fleiss, Joseph L. 1971. "Measuring Nominal Scale Agreement among Many
+    Raters." Psychological Bulletin 76 (5): 378-82.
+    https://doi.org/10.1037/h0031619.
+
+    Randolph, Justus J. 2005 "Free-Marginal Multirater Kappa (multirater
+    K [free]): An Alternative to Fleiss' Fixed-Marginal Multirater Kappa."
+    Presented at the Joensuu Learning and Instruction Symposium, vol. 2005
+    https://eric.ed.gov/?id=ED490661
+
+    Warrens, Matthijs J. 2010. "Inequalities between Multi-Rater Kappas."
+    Advances in Data Analysis and Classification 4 (4): 271-86.
+    https://doi.org/10.1007/s11634-010-0073-4.
+
+    """
 
-    '''
     table = 1.0 * np.asarray(table)   #avoid integer division
     n_sub, n_cat =  table.shape
     n_total = table.sum()
@@ -228,7 +258,10 @@ def fleiss_kappa(table):
     p_rat = (table2.sum(1) - n_rat) / (n_rat * (n_rat - 1.))
     p_mean = p_rat.mean()
 
-    p_mean_exp = (p_cat*p_cat).sum()
+    if method == 'fleiss':
+        p_mean_exp = (p_cat*p_cat).sum()
+    elif method.startswith('rand') or method.startswith('unif'):
+        p_mean_exp = 1 / n_cat
 
     kappa = (p_mean - p_mean_exp) / (1- p_mean_exp)
     return kappa
@@ -394,8 +427,7 @@ def cohens_kappa(table, weights=None, return_results=True, wt=None):
                     kappa_max=kappa_max,
                     weights=weights,
                     var_kappa=var_kappa,
-                    var_kappa0=var_kappa0
-                    )
+                    var_kappa0=var_kappa0)
         return res
     else:
         return kappa
@@ -462,7 +494,7 @@ class KappaResults(ResultsBunch):
     template = _kappa_template
 
     def _initialize(self):
-        if not 'alpha' in self:
+        if 'alpha' not in self:
             self['alpha'] = 0.025
             self['alpha_ci'] = _int_ifclose(100 - 0.025 * 200)[1]
 
@@ -484,4 +516,3 @@ class KappaResults(ResultsBunch):
 
     def __str__(self):
         return self.template % self
-
